#!/usr/bin/env python3
import sys
import time
import serial
import crcmod
import rrdtool
import paramiko

from config import *

calc_crc = crcmod.predefined.mkCrcFun(CRC_TYPE)

def parse_line(line):
    if line[-2:] != "\r\n":
        return {key : None for key in FORMAT}
    line = line[:-2]

    crc_str = line[-2:]
    payload = line[0:-2]
    crc = (ord(crc_str[0]) << 8) | ord(crc_str[1])
    if crc != calc_crc(payload.encode('ascii')):
        return {key : None for key in FORMAT}

    parts = payload.split(';')
    parts = [p.strip() for p in parts]
    parts = [x if x != '#' else None for x in parts]
    data = zip(FORMAT, parts)

    return dict(data)

def create_database():
    sources = []
    for name, minval, maxval in STORED_VALUES:
        sources += ["DS:%s:GAUGE:%d:%f:%f" % (name, DATA_INTERVAL * 2, minval, maxval)]

    now = time.time()
    now = now - (now % DATA_INTERVAL)

    rrd_params = [DATA_FILE,
                    "--start", "%d" % now,
                    "--step", str(DATA_INTERVAL)]
    rrd_params += sources
    rows = ARCHIVE_INTERVAL / DATA_INTERVAL
    rrd_params += ["RRA:LAST:0.1:1:%d" % (rows,)]
    rrdtool.create(*rrd_params)


    rrd_archive_params = [ARCHIVE_DATA_FILE,
                    "--start", "now",
                    "--step", str(DATA_INTERVAL)]
    steps = ARCHIVE_INTERVAL / DATA_INTERVAL
    rows = ARCHIVE_KEEP_INTERVAL / ARCHIVE_INTERVAL
    rrd_archive_params += sources
    rrd_archive_params += ["RRA:AVERAGE:%f:%d:%d" % (MAX_MISSING, steps, rows)]
    rrdtool.create(*rrd_archive_params)



def update_database(line):
    update_values = []
    for name, _, _ in STORED_VALUES:
        update_values += ["%f" % float(line[name])]

    now = time.time()
    now = now - (now % DATA_INTERVAL)
    line = ("%d:" % now) + ":".join(update_values)
    print(line)
    rrdtool.update(DATA_FILE, line)
    rrdtool.update(ARCHIVE_DATA_FILE, line)

def update_graphs():
    for name, lines in GRAPHS.items():

        # Render current data
        graph_params = ['%s.png' % name, '-a', 'PNG', '-s', 'n-%d' % ARCHIVE_INTERVAL]
        for name, lable, color in lines:
            graph_params += ['DEF:%s=%s:%s:LAST' % (name, DATA_FILE, name)]
            graph_params += ['LINE1:%s%s:%s' % (name, color, lable)]

        rrdtool.graph(*graph_params)

        # Also render Archives
        graph_params = ['%s_archive.png' % name, '-a', 'PNG', '-s', 'n-%d' % ARCHIVE_KEEP_INTERVAL]

        for name, lable, color in lines:
            graph_params += ['DEF:%s=%s:%s:AVERAGE' % (name, DATA_FILE, name)]
            graph_params += ['LINE1:%s%s:%s' % (name, color, lable)]

        rrdtool.graph(*graph_params)


def upload_graphs():
    try:
        key = paramiko.RSAKey.from_private_key_file(SFTP_KEY)
        transport = paramiko.Transport((SFTP_HOST, SFTP_PORT))
        transport.connect()
        transport.auth_publickey(SFTP_USER, key)
        sftp = paramiko.SFTPClient.from_transport(transport)

        for name, _ in GRAPHS.items():
            sftp.put('%s.png' % name, 'solar/%s.png' % name)
            sftp.put('%s_archive.png' % name, 'solar/%s_archive.png' % name)

        sftp.close()
        transport.close()
    except:
        print("Unexpected error uploading:", sys.exc_info()[1])

def main():
    pass


if __name__ == '__main__':
    main()
